
package org.objectweb.fractal.bf.hdl.graph;

import java.util.LinkedHashSet;
import java.util.List;


import org.objectweb.fractal.bf.hdl.reader.PairOfIps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import jdsl.graph.api.Edge;
import jdsl.graph.api.Graph;
import jdsl.graph.api.Vertex;



/**
 * A path is a (ordered) sequence of vertex identifiers.
 */
public class Path
{
  Logger logger = LoggerFactory.getLogger(Path.class);
  
  List<Edge> edgesInPath;
  LinkedHashSet<String> verticesInPath;
  LinkedHashSet<String> ipsInPath;
  private Graph graph;
  
  private Integer weight = 0; //default value

  /**
   * @param pov2
   * @param edges
   * @param graph
   */
  public Path(Graph graph, List<Edge> edges)
  {
   
    this.edgesInPath = edges;
    this.graph = graph;
    this.verticesInPath = new LinkedHashSet<String>();
    this.ipsInPath = new LinkedHashSet<String>();
    initializeVerticesInPath();
  }
  
  /**
   * Iterate over edged in path
   */
  private void initializeVerticesInPath()
  {
    

    for (Edge e : edgesInPath)
    {
      Vertex[] connectingVerteces = this.graph.endVertices(e);
      final Vertex firstVertex = connectingVerteces[0];
      verticesInPath.add((String)firstVertex.element());
      final Vertex secondVertex = connectingVerteces[1];
      verticesInPath.add((String)secondVertex.element());
      
      //update the weight 
      logger.debug("Edge details: "+e);
      Integer latency = (Integer) e.get("latency");
      logger.debug("Edge latency: "+latency);
      this.weight = this.weight+latency;
      
      //get the IPs decorating this edge and add them to the list of ipsInPath
      PairOfIps poi = (PairOfIps) e.get("poi");
      this.ipsInPath.add(poi.getFrom_ip());
      this.ipsInPath.add(poi.getTo_ip());
    }

    logger.debug("Vertices in shortest path: " + verticesInPath.toString());
  }

  /**
   * @return
   */
  public Integer countVerticesInPath()
  {
   return verticesInPath.size();
  }

  /**
   * @return
   */
  public Integer weight()
  {
    return this.weight;
  }

  /**
   * Find the position of the specified vertex id
   * @param vertexId
   * @return
   */
  public Integer position(String vertexId)
  {
    Integer position = -1;
    if (this.verticesInPath.contains(vertexId)) {
      Integer current = 0;
      for (String vid : verticesInPath) {
        if (vid.equalsIgnoreCase(vertexId)) {
          position = current;
          break;
        }else {
          current++;
        }
      }
    }
    return position;
  }

  /*
   * TODO
  * successorOf and ipSuccessorOf shares a lot of common code. they iterates over a different set, and 
  * a refactoring is envisioned to reduce this code...
  */
  
  /**
   * @param vertexId
   * @return
   */
  public String successorOf(String vertexId)
  {
    Integer positionOfVertexId = positionOfElementInSet(vertexId, this.verticesInPath);
    Integer positionOfSuccessor = positionOfVertexId + 1;
    String[] verticesInPathAsArray = verticesInPath.toArray(new String[]{});
    return verticesInPathAsArray[positionOfSuccessor];
  }

  /**
   * Find the successor of the specified IP address (as string) along this path
   * 
   * @param ip an IP address in this path
   * @return the successor
   */
  public String ipSuccessorOf(String ip)
  {
    Integer positionOfIp = positionOfElementInSet(ip, this.ipsInPath);
    Integer positionOfSuccessor = positionOfIp + 1;
    String[] ipsInPathAsArray = ipsInPath.toArray(new String[]{});
    return ipsInPathAsArray[positionOfSuccessor];
  }
  
  public String ipAncestorOf(String ip)
  {
    Integer positionOfIp = positionOfElementInSet(ip, this.ipsInPath);
    Integer positionOfSuccessor = positionOfIp -1;
    String[] ipsInPathAsArray = ipsInPath.toArray(new String[]{});
    return ipsInPathAsArray[positionOfSuccessor];
  }

  public final LinkedHashSet<String> getIpsInPath()
  {
    return ipsInPath;
  }

  /**
   * Find the position of the specified element in the given set, or -1 otherwise.
   * @param element
   * @param set
   * @return
   */
  private Integer positionOfElementInSet(String element,
      LinkedHashSet<String> set)
  {

    Integer position = -1;
    if (set.contains(element))
    {
      Integer current = 0;
      for (String vid : set)
      {
        if (vid.equalsIgnoreCase(element))
        {
          position = current;
          break;
        }
        else
        {
          current++;
        }
      }
    }
    return position;
  }

  @Override
  public String toString()
  {
    return this.ipsInPath.toString();
  }
 
  
}
